﻿using Microsoft.EntityFrameworkCore;
using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.SeatchModels;
using RenovationWorkContracts.StorageContracts;
using RenovationWorkContracts.ViewModels;
using RenovationWorkDatabaseImplement.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RenovationWorkDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new RenovationWorkDatabase();

            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();

                return GetViewModel(element);
            }

            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new RenovationWorkDatabase();

            return context.Orders
                .Include(x => x.Repair)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .FirstOrDefault(x => (model.Status == null || model.Status != null && model.Status == x.Status) &&
                        model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId ||
                        model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new RenovationWorkDatabase();
            if (model.Id.HasValue)
            {
                return context.Orders
                    .Include(x => x.Repair)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.DateFrom != null && model.DateTo != null)
            {
                return context.Orders
                    .Include(x => x.Repair)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Repair)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ImplementerId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Repair)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.ImplementerId == model.ImplementerId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Orders
                .Include(x => x.Repair)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Where(x => model.Status == x.Status)
                .Select(x => x.GetViewModel)
                .ToList();

        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new RenovationWorkDatabase();
            return context.Orders
                .Include(x => x.Repair)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }

            using var context = new RenovationWorkDatabase();

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                         .Include(x => x.Repair)
                         .Include(x => x.Client)
                         .FirstOrDefault(x => x.Id == newOrder.Id)
                         ?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new RenovationWorkDatabase();

            var order = context.Orders.Include(x => x.Client).FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders
                          .Include(x => x.Repair)
                          .Include(x => x.Client)
                          .FirstOrDefault(x => x.Id == model.Id)
                          ?.GetViewModel;
        }

        private static OrderViewModel GetViewModel(Order order)
        {
            var viewModel = order.GetViewModel;
            using var context = new RenovationWorkDatabase();
            var element = context.Repairs.FirstOrDefault(x => x.Id == order.RepairId);
            viewModel.RepairName = element.RepairName;
            return viewModel;
        }
    }
}
