﻿using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.ViewModels;
using RenovationWorkDataModels.Enums;
using RenovationWorkDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RenovationWorkListImplement.Models
{
    public class Order : IOrderModel
    {
        public int RepairId { get; private set; }
        public string? RepairName { get; private set; }
        
        public int Count { get; private set; }

        public double Sum { get; private set; }
        public int ClientId { get; private set; }

        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public int Id { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                RepairId = model.RepairId,
                RepairName = model.RepairName,
                Count = model.Count,
                Sum = model.Sum,
                ClientId = model.ClientId,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            RepairId = model.RepairId;
            RepairName = model.RepairName;
            Count = model.Count;
            Sum = model.Sum;
            ClientId = model.ClientId;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId = RepairId,
            RepairName = RepairName,
            Count = Count,
            ClientId = ClientId,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        public int? ImplementerId { get; private set; }
    }
}

